



Rem	DOCUMENTATION
	
	
	Note that color 255 (named TRANSPARENT by the source file PalletConstants.bmx) is essentially treated as a nonexistant
	color. A tile itself cannot possess the color without special intervention - if you want a tile not to be drawn, set it to
	what you've specified to be the tilemap's transparentcolor. The purpose of the color TRANSPARENT is to leave a tile's
	original color intact while still changing its other parameters.
	
		
	InitGraphics:TGraphics ( file$ , windowtitle$ = "?USE_DEFAULT" )
		Create a graphics object according to the settings described in file$. You can override any window title set in 
		file$ using windowtitle$.
	
	tilemap.transparentcolor@ = BLACK
		If the foreground or background of a tile is this color, it's not drawn and rather assumed to be the same color as 
		what Cls produces
	
	tilemap.Create ( width% , height% )
		Create a tilemap object
		
	tilemap.CopyRect ( source:tilemap , source_x% , source_y% , width% , height% , destination:tilemap , destination_x% , 
			      destination_y% )
		Copies tile data from one place to another.
		
	Field transparentcolor@ = BLACK
		When a new tilemap is created, its transparentcolor is set to tilemap.defaulttransparentcolor; this variable allows 
		you to change each tilemap's transparent color on an individual basis.
		
	Method InsertMapImage ( source:tilemap , x% , y% )
		A more friendly way to CopyRect the entirety of one tilemap onto another
		
	Method Fill ( char% , foreground% , background% = BLACK )
		Fill an entire tilemap with a specific tile
		
	Method FillColor ( foreground% , background% = TRANSPARENT )
		Fill an entire tilemap with specific colors.
		
	Method DrawWindow ( title$ , x% , y% , width% , height% )
		Draws a window border on a tilemap without worrying about using a gadget
		
		
		
		
		
		
		
		
	
EndRem






SuperStrict 

Import brl.linkedlist
Import brl.pngloader
Import brl.polledinput
Import brl.blitz
Import brl.GLGraphics
Import brl.glmax2d
Import brl.filesystem
Import brl.stream
Import brl.random
Import brl.math
Import "parse.bmx"
'Import "palletconstants.bmx"



Const newlinechar@=255
Const NEWLINE$=Chr(newlinechar)


Global frameskip%=0

Global gfxscale%=1


'graphics code

Global settingsread%=False
Global defaulttilesetpath$="data\image\curses.png"
Global defaultpalletpath$="data\image\pallet.png"
Global defaultwindowtitle$="World's End"
Global gtilesw%=96,gtilesh%=32
Global gw%,gh%
Function ReadGraphics(path$)
	Local file:TStream=ReadFile(path)
	If Not file Then RuntimeError "Failed to read "+path
	Local online%=0
	Repeat
		online:+1
		Local line$=ReadLine(file)
		If Len(line)>3 And Left(line,2)<>"//" Then
			Local command$=Parse(line,"on line "+online+" of "+path)
			Select Lower(command)
				Case "width"   gtilesw=Int(param[0])
				Case "height"  gtilesh=Int(param[0])
				Case "tileset" defaulttilesetpath=param[0]
				Case "pallet"  defaultpalletpath=param[0]
				Case "title"   defaultwindowtitle=param[0]
				Case "scale"   gfxscale=Int(param[0])
				Default RuntimeError "Unrecognized keyword '"+command+"' on line "+online+" of "+path
			End Select
			If numparams<>1 Then RuntimeError "Incorrect number of arguments on line "+online+" of "+path
		EndIf
	Until Eof(file)
	settingsread=True
End Function

Function InitGraphics:TGraphics(settingspath$,windowtitle$="?USE_DEFAULT")
	If FileType(settingspath)=1 Then 
		ReadGraphics(settingspath)
	ElseIf settingsread=False
		RuntimeError "Failed to read "+settingspath
	EndIf
	If windowtitle="?USE_DEFAULT" windowtitle=defaultwindowtitle
	LoadTileset
	ReadPallet
	gw=gtilesw*tilew*gfxscale
	gh=gtilesh*tileh*gfxscale
	AppTitle=windowtitle
	Local graphicsobject:TGraphics=Graphics(gw,gh)
	Return graphicsobject
End Function

'image to use for characters
Global tsetforeground:TImage
Global tsetbackground:TImage
Global tilew@,tileh@ 'dimensions of an individual ascii tile
Function LoadTileset(path$="?USE_DEAFULT")
	If path="?USE_DEAFULT" path=defaulttilesetpath
	path=StripExt(path)
	Local getinfo:TImage=LoadImage(path+"_fg.png")
	If Not getinfo Then RuntimeError "Failed to load "+path+"_fg.png"
	SetMaskColor 0,0,0
	tsetforeground=LoadAnimImage(path+"_fg.png",ImageWidth(getinfo)/16,ImageHeight(getinfo)/16,0,256,maskedimage)
	If Not tsetforeground Then RuntimeError "Failed to load tileset image "+path+"_fg.png"
	tsetbackground=LoadAnimImage(path+"_bg.png",ImageWidth(getinfo)/16,ImageHeight(getinfo)/16,0,256,maskedimage)
	If Not tsetbackground Then RuntimeError "Failed to load tileset image "+path+"_bg.png"
	tilew=ImageWidth(getinfo)/16
	tileh=ImageHeight(getinfo)/16
End Function

'pallet
Global palletrgb@[256*3] 'contains 256 r/g/b values
Function SetPalletColor(index%) 'set the 
	SetColor palletrgb[index],palletrgb[index+256],palletrgb[index+512]
End Function
Function ReadPallet(path$="?USE_DEAFULT") 'read the pallet from a file
	If path="?USE_DEAFULT" path=defaultpalletpath
	Local source:TPixmap=LoadPixmap(path)
	If Not source RuntimeError "Failed to load "+path
	Local index%=0
	For Local y%=0 To PixmapHeight(source)-1
	For Local x%=0 To PixmapWidth(source)-1
		Local value%=ReadPixel(source,x,y)
		palletrgb[index+512]=value Mod 256
		palletrgb[index+256]=Floor(value/256.0) Mod 256
		palletrgb[index    ]=Floor(value/65536.0) Mod 256
		index:+1
		If index>256 Then RuntimeError "Too many colors in pallet "+path+", 256 is the proper amount."
	Next
	Next
End Function
Function PalletGray%(index%) 'returns (r+g+b)/3 of a pallet index
	Return (palletrgb[index]+palletrgb[index+256]+palletrgb[index+512])/3
End Function


Function RectsOverlap%(x1%,y1%,w1%,h1%,x2%,y2%,w2%,h2%)
	If x2>x1 And x2-x1>w1 Return 0
	If x1>x2 And x1-x2>w2 Return 0
	If y2>y1 And y2-y1>h1 Return 0
	If y1>y2 And y1-y2>h2 Return 0
	Return 1
End Function


'particles that pollute the tilemap
Type tparticle
	Field x#,y#,xv#,yv#
	Field blinkrate% '# of frames between toggle on/off. 0 means it does not blink
	Field life% 'remaining frames until removal
	Field char@,color@,bgcolor@
End Type


Type vector
	Field x%,y%
	Function Create:vector(x%,y%)
		Local v:vector=New vector
		v.x=x;v.y=y
		Return v
	End Function
End Type


'tilemap to draw
Type tilemap
	Global defaulttransparentcolor@=BLACK
	Field drawtime%=0
	Field tmapw%=0,tmaph%=0 'tilemap dimensions
	Field tmap@[tmapw*tmaph*3] 'tilemap array - x,y,tile/color/bgcolor
	Field tplist:TList 'tparticles contained within this tilemap
	
	Field transparentcolor%=BLACK 'don't draw fg or bg of this color on top of existing pixels on the buffer
	
	Function Create:tilemap(width%,height%) 'create a new, blank tilemap
		Local this:tilemap=New tilemap
		this.transparentcolor=defaulttransparentcolor
		this.tmapw=width;this.tmaph=height
		this.tmap=this.tmap[..this.tmapw*this.tmaph*3]
		this.tplist=CreateList()
		Return this
	End Function
	
	Function CopyRect(source:tilemap,sx%,sy%,w%,h%,dest:tilemap,dx%=0,dy%=0,allowtransparent%=0)
		If source=dest And RectsOverlap(sx,sy,w,h,dx,dy,w,h) Then 'do it a bit differently in case the source and destination data overlap			Local store:tilemap=tilemap.Create(w,h)
			Local store:tilemap=tilemap.Create(w,h)
			CopyRect source,sx,sy,w,h,store,0,0,allowtransparent
			CopyRect store,0,0,w,h,source,dx,dy,allowtransparent
			Return
		Else
			Local schar@,sfg@,sbg@
			For Local copyx%=0 To w-1
			For Local copyy%=0 To h-1
				source.grabtile copyx+sx,copyy+sy,schar,sfg,sbg
				dest.settile copyx+dx,copyy+dy,schar,sfg,sbg,allowtransparent
			Next
			Next
			Return
		EndIf
	End Function
	
	
	Method FloodFill%(onmap:tilemap,x%,y%,char@,color@=TRANSPARENT,bgcolor@=TRANSPARENT,filldiagonal%=0,bleed%=0,allowtransparent%=0)
		Local filled%[tmapw,tmaph]
		Local ret_tilesfilled%=0
		Local tofill:TList=CreateList()
		Local newrecent:TList=CreateList()
		Local targc@,targf@,targb@
		grabtile x,y,targc,targf,targb
		'If targc=char And targf=color And targb=bgcolor Then Return -1
		tofill.addlast vector.Create(x,y)
		Repeat
			For Local v:vector=EachIn tofill
				tofill.remove v
				For Local xx%=-1 To 1
				For Local yy%=-1 To 1
					If v.x+xx=>0 And v.x+xx<tmapw And v.y+yy=>0 And v.y+yy<tmaph Then
						If (Abs(xx)+Abs(yy)-1)<=filldiagonal And (Abs(xx)+Abs(yy))>0 And filled[v.x+xx,v.y+yy]=0 Then
							Local grabc@,grabf@,grabb@
							grabtile v.x+xx,v.y+yy,grabc,grabf,grabb
							Local dofill%=(grabc=targc) And (grabf=targf) And (grabb=targb)
							If bleed Then
								onmap.settile v.x+xx,v.y+yy,char,color,bgcolor,allowtransparent
							EndIf
							If dofill Then
								For Local f:vector=EachIn tofill
									If f.x=v.x+xx And f.y=v.y+yy Then dofill=0;Exit
								Next
								If dofill Then
									For Local o:vector=EachIn tofill
										If o.x=v.x+xx And o.y=v.y+yy Then dofill=0;Exit
									Next
									If dofill Then
										tofill.addlast vector.Create(v.x+xx,v.y+yy)
										onmap.settile v.x+xx,v.y+yy,char,color,bgcolor,allowtransparent
										filled[v.x+xx,v.y+yy]=1
										ret_tilesfilled%:+1
									EndIf
								EndIf
							EndIf
						EndIf
					EndIf
				Next
				Next
			Next
			If tofill.count()=0 Then Exit
		Forever
		Return ret_tilesfilled%
	End Method
	
	
	Method InsertCharTable(x%=0,y%=0)
		For Local fillchar%=0 To 255
			settile x+fillchar Mod 16,y+Int(fillchar/16.0),fillchar
		Next
	End Method
	Method InsertColorTable(x%=0,y%=0)
		For Local fillchar%=0 To 255
			settile x+fillchar Mod 16,y+Int(fillchar/16.0),0,TRANSPARENT,fillchar
		Next
	End Method
	
	Method InsertMapImage(source:tilemap,x%,y%,allowtransparent%=0)
		CopyRect source,0,0,source.tmapw,source.tmaph,Self,x,y,allowtransparent
	End Method
	
	Method Fill%(char%,color%,bgcolor%=BLACK,allowtransparent%=0) 'set all tile data
		If char=TRANSPARENT And color=TRANSPARENT And bgcolor=TRANSPARENT And allowtransparent=0 Then Return 0
		For Local x%=0 To tmapw-1
		For Local y%=0 To tmaph-1
			If char<>TRANSPARENT    Or allowtransparent tmap[((x)+(y*tmapw))]                =char
			If color<>TRANSPARENT   Or allowtransparent tmap[((x)+(y*tmapw))+(tmapw*tmaph)]  =color
			If bgcolor<>TRANSPARENT Or allowtransparent tmap[((x)+(y*tmapw))+(tmapw*tmaph*2)]=bgcolor
		Next
		Next
		Return 1
	End Method
	Method FillColor%(color%,bgcolor%=TRANSPARENT,allowtransparent%=0)
		If color=TRANSPARENT And bgcolor=TRANSPARENT And allowtransparent=0 Then Return 0
		For Local x%=0 To tmapw-1
		For Local y%=0 To tmaph-1
			If color<>TRANSPARENT   Or allowtransparent tmap[((x)+(y*tmapw))+(tmapw*tmaph)]  =color
			If bgcolor<>TRANSPARENT Or allowtransparent tmap[((x)+(y*tmapw))+(tmapw*tmaph*2)]=bgcolor
		Next
		Next
		Return 1
	End Method
	
	Method DrawWindow:gadget(title$,x%,y%,w%,h%)
		Local temp:gadget=gadget.CreateWindow(title,x,y,w,h)
		temp.draw Self
		Return temp
	End Method
	Method DrawLabel:gadget(title$,x%,y%,w%,h%)
		Local temp:gadget=gadget.CreateLabel(title,x,y,w,h)
		temp.draw Self
		Return temp
	End Method
	
	Method AddParticle:tparticle(x#,y#,xv#,yv#,blinkrate%,life%,char@,color@,bgcolor@=TRANSPARENT) 'add a particle to the tilemap
		Local part:tparticle=New tparticle
		part.x=x;part.y=y;part.xv=xv;part.yv=yv;part.blinkrate=blinkrate;part.life=life;part.char=char;part.color=color;part.bgcolor=bgcolor
		tplist.addlast part
		Return part
	End Method
	
	Method Copy:tilemap() 'return a duplicate of this tilemap
		Local dest:tilemap
		dest=New tilemap
		dest.tmapw=tmapw;dest.tmaph=tmaph
		dest.tmap=dest.tmap[..tmapw*tmaph*3]
		For Local copyindex%=0 To tmapw*tmaph*3-1
			dest.tmap[copyindex]=tmap[copyindex]
		Next
		dest.drawtime=drawtime
		dest.transparentcolor=transparentcolor
		dest.tplist=CreateList()
		If tplist.count()>0 Then
			For Local this:tparticle=EachIn tplist
				Local that:tparticle=New tparticle
				that.x=this.x;that.y=this.y;that.xv=this.xv;that.blinkrate=this.blinkrate;that.life=this.life;that.char=this.char;that.color=this.color;that.bgcolor=this.bgcolor
				dest.tplist.addlast that
			Next
		EndIf
		Return dest
	End Method
	
	Method GrabTile(x%,y%,char@ Var,color@ Var,bgcolor@ Var)
		If x<0 Or x=>tmapw Or y<0 Or y=>tmaph Then Return
		char=		tmap[((x)+(y*tmapw))]   
		color=	tmap[((x)+(y*tmapw))+(tmapw*tmaph)]
		bgcolor=	tmap[((x)+(y*tmapw))+(tmapw*tmaph*2)]
		Return
	End Method
	Method GrabTileInt(x%,y%,char% Var,color% Var,bgcolor% Var)
		If x<0 Or x=>tmapw Or y<0 Or y=>tmaph Then Return
		char=		tmap[((x)+(y*tmapw))]   
		color=	tmap[((x)+(y*tmapw))+(tmapw*tmaph)]
		bgcolor=	tmap[((x)+(y*tmapw))+(tmapw*tmaph*2)]
		Return
	End Method
	
	Method SetTile%(x%,y%,char@,color@=TRANSPARENT,bgcolor@=TRANSPARENT,allowtransparent%=0) 'set an individual tile's contents
		If x=>0 And y=>0 And x<tmapw And y<tmaph Then
			If char<>TRANSPARENT    Or allowtransparent tmap[((x)+(y*tmapw))]                =char
			If color<>TRANSPARENT   Or allowtransparent tmap[((x)+(y*tmapw))+(tmapw*tmaph)]  =color
			If bgcolor<>TRANSPARENT Or allowtransparent tmap[((x)+(y*tmapw))+(tmapw*tmaph*2)]=bgcolor
		Else
			Return 0 'returns 0 if set tile was out of bounds
		EndIf
		Return 1 'return 1 if tile was successfully set
	End Method
	Method SetTiles%(x%,y%,w%,h%,char@,color@=TRANSPARENT,bgcolor@=TRANSPARENT,allowtransparent%=0)
		For Local setx%=x To x+w-1
		For Local sety%=y To y+h-1
			SetTile setx,sety,char,color,bgcolor,allowtransparent
		Next
		Next
	End Method
	Method SetColors(x%,y%,w%,h%,color@=TRANSPARENT,bgcolor@=TRANSPARENT,allowtransparent%=0)
		If color=TRANSPARENT And bgcolor=TRANSPARENT And allowtransparent=0 Then Return
		For Local setx%=x To x+w-1
		For Local sety%=y To y+h-1
			If x=>0 And y=>0 And x<tmapw And y<tmaph Then
				If color<>TRANSPARENT   Or allowtransparent tmap[((setx)+(sety*tmapw))+(tmapw*tmaph)]  =color
				If bgcolor<>TRANSPARENT Or allowtransparent tmap[((setx)+(sety*tmapw))+(tmapw*tmaph*2)]=bgcolor
			EndIf
		Next
		Next
	End Method
	Method InsertString(x%,y%,text$,color@=TRANSPARENT,bgcolor@=TRANSPARENT,dodcolor%=0,allowtransparent%=0,allownewline%=1)
		Local colorstring$="",coloring%=0,defaultcolor@=color
		Local charx%=x
		For Local set%=0 To Len(text)-1
			Local char%=Asc(Mid(text,set+1,1))
			If char=Asc("@") And dodcolor Then 
				coloring=Not coloring
				If coloring=1 Then color=defaultcolor
			ElseIf char=Asc(NEWLINE) And allownewline
				charx=x;y:+1
			ElseIf coloring And dodcolor
				colorstring:+Chr(char)
				color=Int(colorstring)
			Else
				colorstring=""
				If SetTile(charx,y,char,color,bgcolor,allowtransparent)=0 Then Exit
				charx:+1
			EndIf
		Next
	End Method
	
	Method Save(path$) 'save the tilemap to a file to reload later
		Local file:TStream=WriteFile(path)
		If Not file Then Return'RuntimeError "Failed to write "+path
		Local rle%=rlesize()
		If rle<tmapw*tmaph*3 Or tmapw*tmaph*3<2^32 Then
			WriteRLE file
		ElseIf tmapw*tmaph*3<2^32
			WriteString file,"WETM"
			WriteInt file,tmapw
			WriteInt file,tmaph
			'WriteByte file,transparentcolor
			WriteInt file,GetChecksum()
			For Local loop%=0 To tmapw*tmaph*3-1
				WriteByte file,tmap[loop]
			Next
			WriteString file,"EOF"
		Else
			Return'RuntimeError "Failed to save tilemap "+path
		EndIf
		CloseFile file
	End Method
	Method GetChecksum%() 'retrieves a checksum used to help keep save files secure from hacking
		Local sum1%=0,sum2%=2^20
		For Local loop%=0 To tmapw*tmaph*3-1
			sum1=((sum1+tmap[loop]) Mod 65535)*2
			sum2=Abs(sum2-tmap[loop])*-1
		Next
		Return Int(sum1-sum2)
	End Method
	
	Method WriteRLE(stream:TStream)
		WriteString stream,"WETR"
		WriteInt stream,tmapw
		WriteInt stream,tmaph
		'WriteByte stream,transparentcolor
		WriteInt stream,GetChecksum()
		Local last%=tmap[0],count%=-1
		For Local go%=0 To tmapw*tmaph*3-0
			If tmap[go]<>last Or go=tmapw*tmaph*3-0 Or count=255
				WriteByte stream,count
				WriteByte stream,last
				count=0
				last=tmap[go]
			Else
				count:+1
			EndIf
		Next
		WriteString stream,"EOF"
	End Method
	
	Method rlesize%()
		Local retsize%=0
		Local last%,count%
		For Local go%=0 To tmapw*tmaph*3-0
			If tmap[go]<>last Or go=tmapw*tmaph*3-0 Or count=255
				retsize:+2
				count=0
				last=tmap[go]
			Else
				count:+1
			EndIf
		Next
		Return retsize
	End Method
	
	
	Function Read:tilemap(path$) 'read a previously saved tilemap from a file path
		Local this:tilemap=New tilemap
		this.tplist=CreateList()
		Local file:TStream=ReadFile(path)
		If Not file Return Null
		Local header$=ReadString(file,4)
		this.tmapw=ReadInt(file);this.tmaph=ReadInt(file)
		'this.transparentcolor=ReadByte(file)
		Local checksum%=ReadInt(file)
		If header="WETR"
			this.tmap=this.tmap[..this.tmapw*this.tmaph*3]
			Local total%=0
			Repeat
				Local count@=ReadByte(file),kind@=ReadByte(file)
				For Local fill%=total To total+count
					this.tmap[fill]=kind
				Next
				total:+count+1
				If total=>this.tmapw*this.tmaph*3-0 Then Exit
			Forever
			If Not ReadString(file,3)="EOF" Return Null
			CloseFile file
		ElseIf header="WETM"
			this.tmap=this.tmap[..this.tmapw*this.tmaph*3]
			For Local loop%=0 To this.tmapw*this.tmaph*3-1
				this.tmap[loop]=ReadByte(file)
				If Eof(file) Return Null
			Next
			If Not ReadString(file,3)="EOF" Return Null
			CloseFile file
		Else
			Return Null
		EndIf
		If this.GetChecksum()<>checksum Then Return Null
		Return this
	End Function	
	Rem  -- original read function in all its debuggy goodness
	Function Read:tilemap(path$) 'read a previously saved tilemap from a file path
		Local this:tilemap=New tilemap
		this.tplist=CreateList()
		Local file:TStream=ReadFile(path)
		If Not path Return Null
		If ReadString(file,4)<>"WETM" RuntimeError "Unregonized file header in "+path
		this.tmapw=ReadInt(file);this.tmaph=ReadInt(file)
		transparentcolor=ReadByte(file)
		Local checksum%=ReadInt(file)
		this.tmap=this.tmap[..this.tmapw*this.tmaph*3]
		For Local loop%=0 To this.tmapw*this.tmaph*3-1
			this.tmap[loop]=ReadByte(file)
			If Eof(file) RuntimeError "Encountered unexpected end of file in "+path
		Next
		If Not ReadString(file,3)="EOF" RuntimeError "Did not find expected end of file in "+path
		CloseFile file
		If this.GetChecksum()<>checksum Then RuntimeError "The save file "+path+" has been tampered with; the checksums do not match."
		Return this
	End Function
	EndRem
	
	Method Draw(x%,y%,xoffset%=0,yoffset%=0,tilesw%=-1,tilesh%=-1) 'x,y on screen, x,y offset in tilemap, and w,h of tiles drawn
		SetScale gfxscale,gfxscale
		If tilesw=-1 Then tilesw=tmapw
		If tilesh=-1 Then tilesh=tmaph
		If tilesw>tmapw Then tilesw=tmapw
		If tilesh>tmaph Then tilesh=tmaph
		x:*tilew*gfxscale
		y:*tileh*gfxscale
		If xoffset<0 xoffset=0
		If yoffset<0 yoffset=0
		If xoffset>tmapw-tilesw Then xoffset=tmapw-tilesw
		If yoffset>tmaph-tilesh Then yoffset=tmaph-tilesh
		Local artarray@[tilesw*tilesh*3]
		For Local drawx%=0 To tilesw-1
		For Local drawy%=0 To tilesh-1
			artarray[(drawx+drawy*tilesw)]                  =tmap[((drawx+xoffset)+(drawy+yoffset)*tmapw)]
			artarray[(drawx+drawy*tilesw)+(tilesw*tilesh)]  =tmap[((drawx+xoffset)+(drawy+yoffset)*tmapw)+(tmapw*tmaph)]
			artarray[(drawx+drawy*tilesw)+(tilesw*tilesh*2)]=tmap[((drawx+xoffset)+(drawy+yoffset)*tmapw)+(tmapw*tmaph*2)]
		Next
		Next
		For Local part:tparticle=EachIn tplist
			If part.x=>xoffset And part.y=>yoffset And part.x<xoffset+tilesw And part.y<yoffset+tilesh Then 
				If part.blinkrate=0 Or ((drawtime Mod part.blinkrate*2)<part.blinkrate)
					If part.char<>TRANSPARENT    artarray[((Int(part.x)-xoffset)+(Int(part.y)-yoffset)*tilesw)]                  =part.char
					If part.color<>TRANSPARENT   artarray[((Int(part.x)-xoffset)+(Int(part.y)-yoffset)*tilesw)+(tilesw*tilesh)]  =part.color
					If part.bgcolor<>TRANSPARENT artarray[((Int(part.x)-xoffset)+(Int(part.y)-yoffset)*tilesw)+(tilesw*tilesh*2)]=part.bgcolor
				EndIf
				If part.life=>0 part.life:-1
				part.x:+part.xv;part.y:+part.yv
				If part.life=0 Then tplist.remove part
			EndIf
		Next
		For Local drawx%=0 To tilesw-1
		For Local drawy%=0 To tilesh-1
			If artarray[(drawx+drawy*tilesw)+(tilesw*tilesh*2)]=artarray[(drawx+drawy*tilesw)+(tilesw*tilesh)]
				If artarray[(drawx+drawy*tilesw)+(tilesw*tilesh)]<>transparentcolor And artarray[(drawx+drawy*tilesw)+(tilesw*tilesh)]<>TRANSPARENT Then
					SetPalletColor artarray[(drawx+drawy*tilesw)+(tilesw*tilesh)]
					DrawRect drawx*tilew*gfxscale+x,drawy*tileh*gfxscale+y,tilew,tileh
				EndIf
			Else
				If artarray[(drawx+drawy*tilesw)+(tilesw*tilesh*2)]<>transparentcolor And artarray[(drawx+drawy*tilesw)+(tilesw*tilesh*2)]<>TRANSPARENT Then
					SetPalletColor artarray[(drawx+drawy*tilesw)+(tilesw*tilesh*2)]
					DrawImage tsetbackground,drawx*tilew*gfxscale+x,drawy*tileh*gfxscale+y,artarray[(drawx+drawy*tilesw)]
				EndIf
				If artarray[(drawx+drawy*tilesw)+(tilesw*tilesh)]<>transparentcolor And artarray[(drawx+drawy*tilesw)+(tilesw*tilesh)]<>TRANSPARENT Then
					SetPalletColor artarray[(drawx+drawy*tilesw)+(tilesw*tilesh)]
					DrawImage tsetforeground,drawx*tilew*gfxscale+x,drawy*tileh*gfxscale+y,artarray[(drawx+drawy*tilesw)]
				EndIf
			EndIf
		Next
		Next
		drawtime:+1
		SetScale 1,1
	End Method
	
	Method ClearParticles()
		ClearList tplist
	End Method
	
End Type


Global frame%=0
Global cintext$=""
Global cinallowed$="qazwsxedcrfvtgbyhnujmikolpQAZWSXEDCRFVTGBYHNUJMIKOLP1234567890!@#$%^&*()_+|\[];',./{}:<>?`"
Function cin$(intext$="> ",y%=10,color@=WHITE)
	Local msg:tilemap=tilemap.Create(gtilesw-4,1)
	msg.fill 0,WHITE,BLACK
	cintext=""
	FlushKeys()
	Local uscore$="",utime%=0,lastscore$=uscore,usetext$=""
	Repeat
		Cls
		lastscore=uscore
		If utime Mod 80<40 Then uscore="_" Else uscore=""
		utime:+1
		frame:+1
		
		msg.draw 2,2'tilew*2,y*tileh,0,0,msg.tmapw,msg.tmaph
		
		usetext=intext+"@"+color+"@"+cintext
		Local char$=Chr(GetChar())
		If Instr(cinallowed,char) Then cintext:+char
		If Asc(char)=8 And Len(cintext)>0 Then cintext=Left(cintext,Len(cintext)-1);msg.fill 0,WHITE,TRANSPARENT;msg.insertstring msg.tmapw/2-Len(usetext)/2,0,usetext,WHITE,TRANSPARENT,1
		usetext=intext+"@"+color+"@"+cintext
		If Asc(char)=13 Then 
			uscore=""
			msg.fill 0,WHITE,TRANSPARENT;msg.insertstring msg.tmapw/2-(Len(usetext)-Len(uscore))/2,0,usetext,WHITE,1
			msg.draw tilew*2,y*tileh,0,0,msg.tmapw,msg.tmaph
			FlushKeys
			Return cintext
		EndIf
		If Asc(char)>0 Then uscore=""
		usetext=intext+"@"+color+"@"+cintext+"@@"+uscore
		
		If Asc(char)>0 Or uscore<>lastscore Then msg.fill 0,WHITE,TRANSPARENT;msg.insertstring msg.tmapw/2-(Len(usetext)-Len(uscore))/2,0,usetext,WHITE,TRANSPARENT,1
		
		Flip
	Forever
End Function


Global overlay_y%=0
Function TerminalOverlay()
	If Rand(0,9500)=0 Or overlay_y=0 Then overlay_y=Rand(1,78)
	overlay_y:+Rand(1,2)
	For Local ty%=0 To GraphicsHeight()-1
		If (Rand(0,227)=0) Or (Rand(0,2)=0 And (ty Mod Rand(24,25))=overlay_y) Then
			SetColor 0,0,0
			DrawLine 0,ty,GraphicsWidth()-1,ty
		EndIf
	Next
End Function



Type gadget
	Const WINDOW@=0
	Const LABEL@=1
	Const BUTTON@=2
	
	Field bordercolor@=BLUE
	Field TextColor@=WHITE
	Field bgcolor@=TRANSPARENT
	
	Field kind%
	Field x%,y%
	Field w%,h%
	Field child:TList=CreateList()
	Field hidden%=0
	Field title$=""
	
	Field active%=0
	Field nextactive:gadget=Null
	
	
	Method draw(onmap:tilemap,xoffset%=0,yoffset%=0)
		Local dx%=x+xoffset,dy%=y+yoffset
		If Not hidden Then
			If kind=WINDOW' Or kind=BUTTON
				If h=1
					onmap.settile dx,dy,91,bordercolor,bgcolor
					onmap.settile dx+w-1,dy,93,bordercolor,bgcolor
				Else
					onmap.settile dx,dy,214,bordercolor,bgcolor
					onmap.settile dx+w-1,dy,183,bordercolor,bgcolor
					onmap.settile dx,dy+h-1,211,bordercolor,bgcolor
					onmap.settile dx+w-1,dy+h-1,189,bordercolor,bgcolor
					If h>2
						onmap.settiles dx,dy+1,1,h-2,186,bordercolor,bgcolor
						onmap.settiles dx+w-1,dy+1,1,h-2,186,bordercolor,bgcolor
					EndIf
				EndIf
			EndIf
			If title<>""
				Local showtitle$=title
				If kind=WINDOW
					If w>2 onmap.settiles dx+1,dy,w-2,1,0,bordercolor,bordercolor
					If w>4
						If (w-4)<Len(title) Then showtitle=Left(showtitle,w-4)
						onmap.insertstring dx+2,dy,showtitle,TextColor,TRANSPARENT
					EndIf
				EndIf
				If kind=LABEL
					If Len(title)=<w And Not Instr(title,NEWLINE) Then
						onmap.insertstring dx,dy,showtitle,TextColor,bgcolor
					Else
						title:+" h"
						Local pos%=1,online%=0
						Local line$="",word$="",char$=""
						Repeat
							If online>h Then Exit
							char=Mid(title,pos,1)
							If char=" " Or char=NEWLINE Or pos=Len(title)
								If Len(word)>w Then
									If Len(line)>0 line:+" "
									Local linelen%=Len(line)
									line:+Left(word,w-linelen-1)+"-"
									word=Right(word,Len(word)-Len(Left(word,w-linelen-1)))
									onmap.insertstring dx,dy+online,line,TextColor,bgcolor
									'Print line
									online:+1;line=""
									pos:-1
								Else
									If Len(line)+Len(word)+1=>w Or pos=Len(title) Or char=NEWLINE Then
										If char=NEWLINE line:+" "+word
										onmap.insertstring dx,dy+online,line,TextColor,bgcolor
										'Print line
										online:+1;line=""
									EndIf
									If pos=Len(title) Or char=NEWLINE
										If char<>NEWLINE word:+char
										If Len(line)>0 Then line:+" "
										If pos=Len(title) And Len(line)+Len(word)<=w Then
											line:+word
										ElseIf pos=Len(title)
											onmap.insertstring dx,dy+online,line,TextColor,bgcolor
											online:+1;line=""
											line:+word'onmap.insertstring dx,dy+online,word,TextColor,bgcolor
										EndIf
										word=""
									EndIf
									If Len(line)>0 Then line:+" "
									line:+word
									word=""
								EndIf
							Else
								word:+char
							EndIf
							pos:+1
							If online=>h Or pos>Len(title) Then Exit
						Forever
					EndIf
				EndIf
				If kind=BUTTON
					onmap.insertstring dx,dy,showtitle,TextColor,TRANSPARENT
				EndIf
			EndIf
			For Local kid:gadget=EachIn child
				kid.draw onmap,dx,dy
			Next
		EndIf
	End Method
	
	Function CreateWindow:gadget(title$,x%,y%,w%,h%,parent:gadget=Null)
		Local this:gadget=New gadget
		this.kind=WINDOW
		this.title=title;this.x=x;this.y=y;this.w=w;this.h=h
		If parent Then parent.child.addlast this
		Return this
	End Function
	Function CreateLabel:gadget(title$,x%,y%,w%,h%,parent:gadget=Null)
		Local this:gadget=New gadget
		this.kind=LABEL
		this.title=title;this.x=x;this.y=y;this.w=w;this.h=h
		If parent Then parent.child.addlast this
		Return this
	End Function
	
	Function popup%(title$,disptext$,options:TList) 'options should be a list of strings to show as buttons
		If options.count()<1 Then Return -1
		FlushKeys
		Local win:tilemap=tilemap.Create(38,8)
		win.fill 0,WHITE,BLACK
		win.drawwindow title,0,0,38,8
		win.drawlabel disptext,1,1,36,5
		Local totallen%=options.count()*2
		For Local accu$=EachIn options
			totallen:+Len(accu)
		Next
		Local addx%=0
		For Local add$=EachIn options
			win.insertstring addx*(totallen/options.count())-(totallen/2)+(win.tmapw/2),6,add,WHITE,BCYAN
			addx:+1
		Next
		Local selected%=0
		Repeat
			Cls
			win.draw gtilesw/2-win.tmapw/2,gtilesh/2-win.tmaph/2
			If KeyHit(key_escape) Then FlushKeys;Return options.count()-1 'pressing escape returns the last option.
			If KeyHit(key_enter) Then FlushKeys;Return selected 'pressing enter returns the currently selected option.
			If KeyHit(key_left) Or KeyHit(key_a) Or KeyHit(key_up) Or KeyHit(key_w) Then selected:-1
			If KeyHit(key_right) Or KeyHit(key_d) Or KeyHit(key_tab) Or KeyHit(key_down) Or KeyHit(key_s) Then selected:+1
			While selected<0 
				selected:+options.count()
			Wend
			If selected=>options.count() selected=selected Mod options.count()
			Local underx%=0
			For Local under$=EachIn options
				Local unders$="--"
				If underx=selected Then unders="XX"
				win.insertstring underx*(totallen/options.count())-(totallen/2)+(win.tmapw/2)+Len(under)/2-1,7,unders,DCYAN,BLACK
				underx:+1
			Next
			Flip
		Forever
	End Function
	
End Type


Global notlist:TList=CreateList()
notlist.addlast "Ok"
Function Notify%(title$,text$)
	gadget.popup title,text,notlist
	Return 1
End Function

Global conflist:TList=CreateList()
conflist.addlast "Yes"
conflist.addlast "No"
Function Confirm%(title$,text$)
	Return gadget.popup (title,text,conflist)
End Function


Global mx%,my%,lmbd%,rmbd%,mmbd%,lmbu%,rmbu%,mmbu%
Function UpdateMouse()
	mx=MouseX()/tilew/gfxscale;my=MouseY()/tileh/gfxscale
	Local lastl%=lmbd,lastr%=rmbd,lastm%=mmbd
	lmbd=MouseDown(1)
	rmbd=MouseDown(2)
	mmbd=MouseDown(3)
	If lastl>lmbd Then lmbu=1 Else lmbu=0
	If lastr>rmbd Then rmbu=1 Else rmbu=0
	If lastm>mmbd Then mmbu=1 Else mmbu=0
End Function

Function MouseIn%(x%,y%,w%,h%)
	If mx=>x And mx<x+w And my=>y And my<y+h Then Return 1
	Return 0
End Function


















'constants - white & black, "L"ight shades, "D"ark shades, "B"lack shades, and "W"hite shades
Const WHITE@=0
Const DGRAY@=1
Const LRED@=2
Const LGREEN@=3
Const LBLUE@=4
Const LYELLOW@=5
Const LCYAN@=6
Const LMAGENTA@=7
Const LGRAY@=8
Const BLACK@=9
Const DRED@=10
Const DGREEN@=11
Const DBLUE@=12
Const DYELLOW@=13
Const DCYAN@=14
Const DMAGENTA@=15
Const DWHITE@=16
Const LBLACK@=17
Const BRED@=18
Const BGREEN@=19
Const BBLUE@=20
Const BYELLOW@=21
Const BCYAN@=22
Const BMAGENTA@=23
Const MWHITE@=24
Const BGRAY@=25
Const RED@=26
Const GREEN@=27
Const BLUE@=28
Const YELLOW@=29
Const CYAN@=30
Const MAGENTA@=31
Const ORANGE@=32
Const BROWN@=33
Const WRED@=34
Const WGREEN@=35
Const WBLUE@=36
Const WYELLOW@=37
Const WCYAN@=38
Const WMAGENTA@=39
Const GOLD@=40
Const DVIOLET@=41
Const VIOLET@=42
Const LVIOLET@=43
Const WVIOLET@=44
Const DAQUA@=45
Const AQUA@=46
Const LAQUA@=47

Const TRANSPARENT@=255
